@@ -34,12 +34,11 @@ foreach my $name (@zones) {
     push @clocks, [$name, $time];
 }
 
-# sort clocks based on the local date and time of day
-for my $clock (sort {     $a->[1]->local_rd_as_seconds 
-                      <=> $b->[1]->local_rd_as_seconds } @clocks) {
+for my $clock (sort { $a->[1]->offset <=> $b->[1]->offset } @clocks) {
     my ($name, $time) = @$clock;
-    my $diff = ($time->local_rd_as_seconds - $now->local_rd_as_seconds) / 60;
+    my $diff = ($time->offset - $now->offset);
     printf "%-12s %-16s %s\n", $name, $time->strftime("%a %H:%M %Z"),
       $diff ? sprintf "(%+dm)", $diff : '';
 }
 
+
