@@ -6,7 +6,7 @@ use Carp        qw[];
 use Time::HiRes qw[];
 
 BEGIN {
-    our $VERSION = '0.05';
+    our $VERSION = '0.06';
     require XSLoader; XSLoader::load(__PACKAGE__, $VERSION);
 }
 
@@ -58,11 +58,6 @@ sub DateTime::__as_Time_Moment {
     return Time::Moment->from_epoch($dt->epoch, $usec, $off);
 }
 
-sub __as_Time_Piece {
-    my ($tm) = @_;
-    return scalar Time::Piece::localtime($tm->epoch);
-}
-
 sub Time::Piece::__as_Time_Moment {
     my ($tp) = @_;
     return Time::Moment->from_epoch($tp->epoch, 0, int($tp->tzoffset / 60));
